/*
a container for storing messages.

Messages can be inserted, looked up, and iterated over.
Since there will be more than one such container in l2simu, it can't
be implemented like the other l2simu "tables": function calls will
need a pointer to an area reserved for the messages.

For the lookup, the user must supply a function which compares two
messages, returning -1, 0, 1 depending on whether the first (stored)
message is less than (-1), equal to (0) or greater than (1) the
supplied message. This can be supplied at initialization time. The
idea is that an incoming message should be associated with a unique
reply message which is contained in the list. Thus, the incoming
message is supplied to the lookup function.  If the comparison
function is null on the intialization call, lookups will be disabled.

Lookup will first look at the current value plus one of "iteration",
if valid. This will result in fast lookup when messages are scanned
in order. Lookup resets iteration to the found message. Note that
this clashes with Iterate.

It will be assumed that the order in which the messages are appended
is the search order.

A doubly linked list seems appropriate.

For now at least, no special memory management. Just use malloc.

A copy of the message given will be made at insert. A pointer to the
message will be output. This pointer should not be deleted.

Call it l2messlist.

An error will result in a fatal call to grr_error.

Add output and input functions.

*/

#ifndef _L2MESSLIST
#define _L2MESSLIST

#include "rps.h"

struct L2MESSWRAP { /* An envelope */
  struct L2MESSWRAP *last;
  struct L2MESSWRAP *next;
  RPS_msg *mess;
};
typedef struct L2MESSWRAP L2MESSWRAP;

typedef int (*L2MESSLIST_COMP)(RPS_msg *, RPS_msg *);

typedef struct L2MESSLIST {
  int numMess;
  L2MESSWRAP *first;
  L2MESSWRAP *last;
  L2MESSWRAP *iteration;
  L2MESSLIST_COMP compFunc;
} L2MESSLIST;



void     l2messlist_Init(L2MESSLIST *, L2MESSLIST_COMP);
void     l2messlist_Append(L2MESSLIST *, RPS_msg*);
RPS_msg *l2messlist_Lookup(L2MESSLIST *, RPS_msg*);
RPS_msg *l2messlist_IterateInit(L2MESSLIST *);
RPS_msg *l2messlist_IterateNext(L2MESSLIST *); 
RPS_msg *l2messlist_GetLast(L2MESSLIST *);

void     l2messlist_Destroy( L2MESSLIST *);
void     l2messlist_Output(L2MESSLIST *, char *to);
void     l2messlist_Input(L2MESSLIST *, char *from);
#endif



