/*;*/
/**  rps definitions.

@name rps.h

It provides data structure declarations and function
prototypes. It further may include files with global
variable definitions (rps_glob.h, rps_sglo.h) and
inline function implementation (rps_fun.h, rps_sfun.h)
and sharc specific declarations (rps_shar.h) depending
on defined macros.

If __ASM__ is defined only macros
needed by assembly functions are produced.

If __UNIX__ is defined no SHARC specific code is
generated (also no BPM dependent stuff).

If __NOINLINE__ is defined no inline functions are
produced. No inline function file is included. There are
some utility functions in this file which are implemented as
static functions nevertheless. Function prototypes for
otherwise provided inline functions are generated as 
normal function prototypes.

If __L2SIMU__ is defined l2simu specific
implementations/definitions are included from rps_fun_l2.h.

If __LINUX__ is defined linux specific
implementations/definitions are included from rps_fun_linux.h.

; */
/*@{*/

#ifndef __RPS__
#define __RPS__

#include "errcls.h"

#ifdef __rpsstuff__
#ifndef __UNIX__
#include "bpm.h"
#include "dcd.h"
#include "utl.h"
#endif
#endif

#ifdef __NOINLINE__
#define inkey static
#else
#define inkey inline static
#endif

#ifndef __ASM__

typedef int RPS_err;

/** RPS process identifier.
@name RPS address

An RPS address specifies in the least significant 2byte 
(byte 0 and 1) the SHARC process the message is to be routed
to - the RPS number. RPS numbers enumerate SHARC Processes 
sequentially. 

Byte 2 specifies a sub-address. Sub address NE 0 can be directed
to different input queues according to a local (to the receiving sharc) table.
Non-existing sub-addresses are interpreted as they wrere 0.


The uppermost byte specifies a RPS domain: 0 is the SHARC world  and 1 is
reserved. Any other RPS domain is routed
to the VME host. A RPS domain is a number of a RPS to RPM bridge. */
/*@{ */

/** RPS address type */
typedef int RPS_addr;

/** RPS domain type */
typedef int RPS_domain;

/** RPS subaddress type */
typedef int RPS_sub_addr;

/** RPS rps address type */
typedef int RPS_rps_addr;

/** extract domain from address*/
inkey RPS_domain RPS_get_domain(RPS_addr a){
  return (RPS_domain)((int)(a)>>24);}

/** put domain into address*/
inkey RPS_addr RPS_put_domain(RPS_addr a, RPS_domain d){
  return (RPS_addr)(((int)d)<<24 | (a & 0xffffff));}

/** extract subaddress from address*/
inkey RPS_sub_addr RPS_get_sub_addr(RPS_addr a){
  return (RPS_sub_addr)(0xff & ((int)a)>>16);}

/** put subaddress into address*/
inkey RPS_addr RPS_put_sub_addr(RPS_addr a, RPS_sub_addr s){
  return (RPS_addr)((0xff00ffff & a) | (0xff&((int)s))<<16);}

/** extract rps address from address*/
inkey RPS_rps_addr RPS_get_rps_addr(RPS_addr a){
  return (RPS_rps_addr)(0xffff & ((int)a));}

/** put rps address into address*/
inkey RPS_addr RPS_put_rps_addr(RPS_addr a, RPS_rps_addr r){
  return (RPS_addr)((0x0000ffff & a) | (0xffff&((int)r)));}

/*@} */

#endif

#define RPS_GET_CLASS(t) ((t)>>16)
#define RPS_PUT_CLASS(t,c) (((c)<<16)|(t&0x0000ffff))

#define RPS_GET_CLASS_asm(c,t) c=FEXT t BY 16:8


#ifndef __ASM__
/*;*/
/** Type and Class of a message.

@name RPS type

The header of each message contains a type field which contains 4byte of 
information. The topmost byte is used as a message class - a group of types.
The message class is used to select message handlers if the user has
installed any. */
/*@{ */

/** RPS type declaration */
typedef int RPS_type;

/** RPS class declaration */
typedef int RPS_class;

/** extract class from type*/
inkey RPS_class RPS_get_class(RPS_type t){
  return (RPS_class)RPS_GET_CLASS((int)t);}

/** put class to type*/
inkey RPS_type RPS_put_class(RPS_type t,RPS_class c){
  return (RPS_type)RPS_PUT_CLASS((int)t,(int)c);}

/*@} */


#endif


#ifndef __ASM__

/** Message header. */

struct RPS_hdr {
  /** data length (excluding header) in units of byte */
  int      length;
  /** destination address    */
  RPS_addr dest;
  /** source address         */
  RPS_addr source;
  /** message type code      */
  RPS_type type;
  /** message identifier     */
  int      ident;
  /** message to be sent after this one has been sent out,
      if it is 0 no further message is send */
  struct RPS_msg *after;
};
typedef struct RPS_hdr RPS_hdr;


#endif

#define RPS_HDR_length 0
#define RPS_HDR_dest   1
#define RPS_HDR_source 2
#define RPS_HDR_type   3
#define RPS_HDR_ident  4
#define RPS_HDR_after  5
#define RPS_HDR_sizeof 6


#ifndef __ASM__
/** RPS message.
Messages are of variable length and have a fixed (by installation) 
maximal length. A message consists out of a header, the data and a trailer.
The header contains the message length, its destination and source,
a message type and an identification. The data field is a multiple of
4byte.
*/

struct RPS_msg{
#ifndef __UNIX__
  /** buffer header of the buffer containing the message data */ 
  DCD_buf buf;
#endif
  /** message header of message */
  RPS_hdr hdr;
};
typedef struct RPS_msg RPS_msg;
#endif

#ifdef __UNIX__
#define RPS_MSG_hdr 0
#else
#define RPS_MSG_buf    0
#define RPS_MSG_hdr    DCD_BUF_sizeof
#endif
#define RPS_MSG_sizeof (RPS_MSG_hdr+RPS_HDR_sizeof)



/** maximal size of an RPS message (the user usable part like in the header) in bytes. */ 
#define RPS_MSG_SIZE 800
#define RPS_MSG_MIN_SIZE 0
/** maximal size of an RPS message in words includeing the header */
#define RPS_MSG_DMA_SIZE ((RPS_MSG_SIZE+(RPS_HDR_sizeof*4)+3)/4)

/* RPS error codes. */

#define RPS_ok 0
#define RPS_ERR_inv_adr RPS_ERBIAS+4
#define RPS_ERR_inv_port RPS_ERBIAS+5
#define RPS_ERR_inv_pos RPS_ERBIAS+6
#define RPS_ERR_msg_not_sent RPS_ERBIAS+7
#define RPS_ERR_lost_buf_found RPS_ERBIAS+8
#define RPS_ERR_inv_class RPS_ERBIAS+9
#define RPS_ERR_inv_queue RPS_ERBIAS+10
#define RPS_ERR_inv_dir RPS_ERBIAS+11
#define RPS_ERR_inv_subaddr RPS_ERBIAS+12
#define RPS_ERR_no_mem RPS_ERBIAS+13
#define RPS_ERR_not_initialized RPS_ERBIAS+14
#define RPS_ERR_initialized RPS_ERBIAS+15
#define RPS_ERR_multicast_failure RPS_ERBIAS+16


#ifndef __ASM__




/* documentation to these is provided in the source code files */

int RPS_init(RPS_addr local, int port_enable_pattern, int flag);
RPS_err RPS_lookup(const char *name, RPS_addr *addr);
RPS_err RPS_register(const char *name, RPS_addr mine);


#ifdef __NOINLINE__
#define statkey
#else
#define statkey static
#endif

statkey void     RPS_free(RPS_msg *mes);
statkey RPS_msg *RPS_alloc(void);

statkey RPS_addr RPS_my_address(void);
statkey RPS_msg *RPS_dup(RPS_msg *mes);
statkey int      RPS_send2(RPS_msg *mes,RPS_msg *sec);
statkey RPS_msg *RPS_wait(void);



#ifdef __UNIX__

int RPS_send(RPS_msg *mes);

#endif


#endif


#ifdef __rpsstuff__
#ifndef __UNIX__

#include "rps_shar.h"

#ifndef __ASM__
statkey void RPS_grant_host(void);
int RPS_set_tab(RPS_pos *tab,int n);
RPS_suda *RPS_assign_sub(RPS_sub_addr s, RPS_qu_num q);
int RPS_add_queue(RPS_sub_addr sub,RPS_qu_num *qn);
int RPS_rmv_queue(RPS_qu_num qn);
#endif
#endif
#endif




#ifndef __ASM__

#ifndef __DEF_GLOBAL__

#ifndef __NOINLINE__
#ifdef __rpsstuff__
#ifndef __UNIX__
#include "rps_sfun.h"
#endif
#endif
#endif
#ifdef __L2SIMU__
#include "rps_fun_l2.h"
#endif

#ifdef __LINUX__
#include "rps_fun_linux.h"
#endif

#ifndef __NOINLINE__
#include "rps_fun.h"
#endif

#endif
#endif

#endif



/*@}*/



































